package testgen

/*
 * Copyright 2011 Kazuhiro Sera.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

import sbt._
import sbt.Keys._
import sbt.{ Test => SbtTest }
import testgen._

object TestgenKeys {
  lazy val testgen = InputKey[Unit]("testgen")
  lazy val testgenEncoding = SettingKey[String]("testgen-encoding")
  lazy val testgenTestTemplate = SettingKey[String]("testgen-test-template")
  lazy val testgenScalaTestMatchers = SettingKey[String]("testgen-scala-test-matchers")
  lazy val testgenWithJUnitRunner = SettingKey[Boolean]("testgen-with-junit-runner")
  lazy val testgenLineBreak = SettingKey[String]("testgen-line-break")
  lazy val testgenDebug = SettingKey[Boolean]("testgen-debug")
}

import TestgenKeys._

object TestgenPlugin extends Plugin {

  val testgenTask = inputTask {
    (taskKey: TaskKey[Seq[String]]) =>
      {
        (taskKey, scalaSource in Compile, scalaSource in SbtTest,
          testgenEncoding, testgenTestTemplate, testgenScalaTestMatchers, testgenWithJUnitRunner,
          testgenLineBreak, testgenDebug) map {
            case (args, srcDir, srcTestDir, encoding, testTemplate, scalaTestMatchers, withJUnitRunner, lineBreak, debug) => {
              val testgenExecutor = new TestgenExecutor(new Config(
                encoding = encoding,
                srcDir = srcDir.getAbsolutePath,
                srcTestDir = srcTestDir.getAbsolutePath,
                testTemplate = new TestTemplate(testTemplate),
                scalaTestMatchers = new ScalaTestMatchers(scalaTestMatchers),
                withJUnitRunner = withJUnitRunner,
                lineBreak = new LineBreak(lineBreak),
                debug = debug
              ))
              args match {
                case Nil => println("Usage: testgen [CLASS/PACAKGE OR PATH]")
                case _ => {
                  val pathOrPackage = args.head
                  val tests = testgenExecutor.generateTests(pathOrPackage)
                  tests match {
                    case Nil => println(ErrorMessage.noTargetsToGenerateFor(pathOrPackage))
                    case allTests => allTests foreach (test => test.createFileIfNotExist())
                  }
                }
              }
            }
          }
      }
  }

  val testgenSettings = inConfig(Compile)(Seq(
    testgen <<= testgenTask,
    testgenEncoding := "UTF-8",
    testgenTestTemplate := "scalatest.FunSuite",
    testgenScalaTestMatchers := "ShouldMatchers",
    testgenWithJUnitRunner := false,
    testgenLineBreak := "CRLF",
    testgenDebug := false
  ))

}
